#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.libya import Libya
from tests.common import CommonCountryTests


class TestLibya(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Libya)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Libya(categories=WORKDAY, years=range(2012, self.end_year)))

    def test_special_holidays(self):
        self.assertHolidayName("عطلة رسمية", "2023-12-10")

    def test_peoples_authority_day(self):
        name = "عید إعلان سلطة الشعب"
        self.assertHolidayName(name, (f"{year}-03-02" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))

    def test_american_forces_evacuation_day(self):
        name = "عيد إجلاء القوات الأمريكية"
        self.assertHolidayName(name, (f"{year}-06-11" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))

    def test_glorious_july_revolution_day(self):
        name = "عيد ثورة يوليو المجيدة"
        self.assertHolidayName(name, (f"{year}-07-23" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))

    def test_great_al_fateh_day(self):
        name = "عيد الفاتح العظيم"
        self.assertHolidayName(name, (f"{year}-09-01" for year in range(self.start_year, 2012)))
        self.assertNoHolidayName(name, range(2012, self.end_year))

    def test_anniversary_of_the_february_17_revolution(self):
        name = "ثورة 17 فبراير"
        self.assertHolidayName(name, (f"{year}-02-17" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_labor_day(self):
        name = "عيد العمال"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_national_environmental_sanitation_day(self):
        name = "يوم وطني للإصحاح البيئي"
        self.assertHolidayName(name, (f"{year}-08-14" for year in range(2022, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2022))

    def test_martyrs_day(self):
        name = "يوم الشهيد"
        self.assertHolidayName(name, (f"{year}-09-16" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_liberation_day(self):
        name = "يوم التحرير"
        self.assertHolidayName(name, (f"{year}-10-23" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_independence_day(self):
        name = "عيد الاستقلال"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_islamic_new_year(self):
        name = "عيد رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2021-08-10",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2012, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2012))
        self.assertWorkdayIslamicNoEstimatedHolidayName(name, range(self.start_year, 2012))
        self.assertNoWorkdayIslamicNoEstimatedHolidayName(name, range(2012, self.end_year))

    def test_prophets_birthday(self):
        name = "ذكرى المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertHolidayName(
            name,
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-31",
            "2025-04-01",
            "2025-04-02",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_day_of_arafah(self):
        name = "يوم عرفة"
        self.assertHolidayName(
            name,
            "2020-07-30",
            "2021-07-19",
            "2022-07-08",
            "2023-06-27",
            "2024-06-15",
            "2025-06-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_syrian_revolution_day(self):
        name = "عيد ثورة سوريا"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-03-08" for year in range(self.start_year, 2012))
        )
        self.assertNoWorkdayHolidayName(name, range(2012, self.end_year))

    def test_anniversary_of_arab_league(self):
        name = "ذكرى إنشاء الجامعة العربية"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-03-22" for year in range(self.start_year, 2012))
        )
        self.assertNoWorkdayHolidayName(name, range(2012, self.end_year))

    def test_british_forces_evacuation_day(self):
        name = "عيد إجلاء القوات البريطانية"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-03-28" for year in range(self.start_year, 2012))
        )
        self.assertNoWorkdayHolidayName(name, range(2012, self.end_year))

    def test_italian_forces_evacuation_day(self):
        name = "عيد إجلاء الطليان"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-10-07" for year in range(self.start_year, 2012))
        )
        self.assertNoWorkdayHolidayName(name, range(2012, self.end_year))

    def test_ashura_day(self):
        name = "عاشوراء"
        self.assertNoHolidayName(name)
        self.assertWorkdayIslamicNoEstimatedHolidayName(
            name,
            "1988-08-22",
            "1989-08-11",
            "1990-08-01",
            "2010-12-16",
            "2011-12-05",
        )
        self.assertWorkdayIslamicNoEstimatedHolidayName(name, range(self.start_year, 2012))
        self.assertNoWorkdayIslamicNoEstimatedHolidayName(name, range(2012, self.end_year))

    def test_isra_and_miraj_day(self):
        name = "ذكرى الإسراء والمعراج"
        self.assertNoHolidayName(name)
        self.assertWorkdayIslamicNoEstimatedHolidayName(
            name,
            "1988-03-15",
            "1989-03-05",
            "1990-02-22",
            "2010-07-09",
            "2011-06-29",
        )
        self.assertWorkdayIslamicNoEstimatedHolidayName(name, range(self.start_year, 2012))
        self.assertNoWorkdayIslamicNoEstimatedHolidayName(name, range(2012, self.end_year))

    def test_night_of_forgiveness(self):
        name = "ليلة النصف من شعبان"
        self.assertNoHolidayName(name)
        self.assertWorkdayIslamicNoEstimatedHolidayName(
            name,
            "1988-04-02",
            "1989-03-22",
            "1990-03-12",
            "2010-07-27",
            "2011-07-16",
        )
        self.assertWorkdayIslamicNoEstimatedHolidayName(name, range(self.start_year, 2012))
        self.assertNoWorkdayIslamicNoEstimatedHolidayName(name, range(2012, self.end_year))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-02-17", "ثورة 17 فبراير"),
            ("2024-04-10", "عيد الفطر"),
            ("2024-04-11", "عيد الفطر"),
            ("2024-04-12", "عيد الفطر"),
            ("2024-05-01", "عيد العمال"),
            ("2024-06-15", "يوم عرفة"),
            ("2024-06-16", "عيد الأضحى"),
            ("2024-06-17", "عيد الأضحى"),
            ("2024-06-18", "عيد الأضحى"),
            ("2024-07-07", "عيد رأس السنة الهجرية"),
            ("2024-08-14", "يوم وطني للإصحاح البيئي"),
            ("2024-09-15", "ذكرى المولد النبوي الشريف"),
            ("2024-09-16", "يوم الشهيد"),
            ("2024-10-23", "يوم التحرير"),
            ("2024-12-24", "عيد الاستقلال"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-02-17", "ثورة 17 فبراير"),
            ("2025-03-31", "عيد الفطر"),
            ("2025-04-01", "عيد الفطر"),
            ("2025-04-02", "عيد الفطر"),
            ("2025-05-01", "عيد العمال"),
            ("2025-06-05", "يوم عرفة"),
            ("2025-06-06", "عيد الأضحى"),
            ("2025-06-07", "عيد الأضحى"),
            ("2025-06-08", "عيد الأضحى"),
            ("2025-06-26", "عيد رأس السنة الهجرية"),
            ("2025-08-14", "يوم وطني للإصحاح البيئي"),
            ("2025-09-04", "ذكرى المولد النبوي الشريف (تقديري)"),
            ("2025-09-16", "يوم الشهيد"),
            ("2025-10-23", "يوم التحرير"),
            ("2025-12-24", "عيد الاستقلال"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-02-17", "Anniversary of the February 17 Revolution"),
            ("2025-03-31", "Eid al-Fitr"),
            ("2025-04-01", "Eid al-Fitr"),
            ("2025-04-02", "Eid al-Fitr"),
            ("2025-05-01", "Labor Day"),
            ("2025-06-05", "Day of Arafah"),
            ("2025-06-06", "Eid al-Adha"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-06-08", "Eid al-Adha"),
            ("2025-06-26", "Islamic New Year"),
            ("2025-08-14", "National Environmental Sanitation Day"),
            ("2025-09-04", "Prophet's Birthday (estimated)"),
            ("2025-09-16", "Martyrs' Day"),
            ("2025-10-23", "Liberation Day"),
            ("2025-12-24", "Independence Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.mali import Mali
from tests.common import CommonCountryTests


class TestMali(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1961, 2050)
        super().setUpClass(Mali, years=years)
        cls.no_estimated_holidays = Mali(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        self.assertHolidayName("Jour de l'An", (f"{year}-01-01" for year in range(1961, 2050)))

    def test_armed_forces_day(self):
        name = "Journée de l'Armée"
        self.assertHolidayName(name, (f"{year}-01-20" for year in range(1965, 2050)))
        self.assertNoHolidayName(name, range(1961, 1965))

    def test_march_26_day(self):
        name = "Journée du 26 mars"
        self.assertHolidayName(name, (f"{year}-03-26" for year in range(1992, 2050)))
        self.assertNoHolidayName(name, range(1961, 1992))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(2006, 2050))
        self.assertNoHolidayName(name, range(1961, 2006))

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in range(1961, 2050)))

    def test_africa_day(self):
        name = "Journée de l'Afrique"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(1965, 2050)))
        self.assertNoHolidayName(name, range(1961, 1965))

    def test_independence_day(self):
        self.assertHolidayName(
            "Fête Nationale de la République du Mali",
            (f"{year}-09-22" for year in range(1961, 2050)),
        )

    def test_christmas_day(self):
        self.assertHolidayName("Fête de Noël", (f"{year}-12-25" for year in range(1961, 2050)))

    def test_prophets_birthday(self):
        name_1 = "Journée du Mawloud"
        self.assertHolidayName(
            name_1,
            self.no_estimated_holidays,
            "2001-06-04",
            "2002-05-24",
            "2003-05-13",
            "2004-05-01",
            "2005-04-21",
        )
        self.assertHolidayName(name_1, self.no_estimated_holidays, range(1961, 2006))
        self.assertNoHolidayName(name_1, self.no_estimated_holidays, range(2006, 2050))

        name_2 = "Journée du Maouloud (Naissance du Prophète)"
        self.assertHolidayName(
            name_2,
            "2020-10-29",
            "2021-10-19",
            "2022-10-09",
            "2023-09-28",
            "2024-09-16",
        )
        self.assertHolidayName(name_2, self.no_estimated_holidays, range(2006, 2050))
        self.assertNoHolidayName(name_2, self.no_estimated_holidays, range(1961, 2006))

    def test_prophet_baptism_day(self):
        name = "Journée du Maouloud (Baptême du Prophète)"
        self.assertHolidayName(
            name,
            "2020-11-05",
            "2021-10-26",
            "2022-10-16",
            "2023-10-05",
            "2024-09-23",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2006, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(1961, 2006))

    def test_eid_al_fitr(self):
        name = "Journée de la Fête du Ramadan"
        self.assertHolidayName(
            name,
            "2021-05-12",
            "2022-05-01",
            "2023-04-21",
            "2024-04-09",
            "2025-03-30",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1961, 2050))

    def test_eid_al_adha(self):
        name = "Journée de la Tabaski"
        self.assertHolidayName(
            name,
            "2021-07-21",
            "2022-07-09",
            "2023-06-28",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1961, 2050))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Jour de l'An"),
            ("2024-01-20", "Journée de l'Armée"),
            ("2024-03-26", "Journée du 26 mars"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-04-09", "Journée de la Fête du Ramadan"),
            ("2024-05-01", "Fête du Travail"),
            ("2024-05-25", "Journée de l'Afrique"),
            ("2024-06-17", "Journée de la Tabaski"),
            ("2024-09-16", "Journée du Maouloud (Naissance du Prophète)"),
            ("2024-09-23", "Journée du Maouloud (Baptême du Prophète)"),
            ("2024-09-22", "Fête Nationale de la République du Mali"),
            ("2024-12-25", "Fête de Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Jour de l'An"),
            ("2025-01-20", "Journée de l'Armée"),
            ("2025-03-26", "Journée du 26 mars"),
            ("2025-03-30", "Journée de la Fête du Ramadan"),
            ("2025-04-21", "Lundi de Pâques"),
            ("2025-05-01", "Fête du Travail"),
            ("2025-05-25", "Journée de l'Afrique"),
            ("2025-06-07", "Journée de la Tabaski"),
            ("2025-09-04", "Journée du Maouloud (Naissance du Prophète) (estimé)"),
            ("2025-09-11", "Journée du Maouloud (Baptême du Prophète) (estimé)"),
            ("2025-09-22", "Fête Nationale de la République du Mali"),
            ("2025-12-25", "Fête de Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-20", "Armed Forces Day"),
            ("2025-03-26", "Martyrs' Day"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-25", "Africa Day"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-09-04", "Prophet's Birthday (estimated)"),
            ("2025-09-11", "Prophet's Baptism (estimated)"),
            ("2025-09-22", "National Day of the Republic of Mali"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_TO_PREV_FRI, SUN_TO_NEXT_MON


class MarshallIslands(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Marshall Islands holidays.

    References:
        * [Public Holiday Act 1988](https://web.archive.org/web/20240722022301/http://rmiparliament.org/cms/images/LEGISLATION/PRINCIPAL/1988/1988-0016/PublicHolidaysAct1988_1.pdf)
        * [P.L. 2003-99](https://web.archive.org/web/20250826051106/http://rmiparliament.org/cms/library/public-laws-by-years/22-2003.html?download=219:p-l-2003-99-public-holidays-march-1st-amendment-act-2003&start=20)
        * [P.L. 2015-38](https://web.archive.org/web/20250826045904/https://www.rmiparliament.org/cms/images/LEGISLATION/PHOCA/P.L._2015/P.L.%202015-38%20Public%20Holiday%20Amendment%20Act%202015.pdf)
        * [Public Holidays Act 1988 (Consolidated)](https://web.archive.org/web/20250826051925/http://rmiparliament.org/cms/images/LEGISLATION/PRINCIPAL/1988/1988-0016/1988-0016_2.pdf)
        * <https://web.archive.org/web/20240613114250/https://rmiparliament.org/cms/component/content/article/14-pressrelease/49-important-public-holidays.html?Itemid=101>
        * <https://web.archive.org/web/20230528174331/http://www.rmiembassyus.org/country-profile>
        * <https://en.wikipedia.org/wiki/Elections_in_the_Marshall_Islands>
        * [Nuclear Victims Remembrance Day](https://web.archive.org/web/20240421213436/https://www.pscrmi.net/_files/ugd/8c401f_ac2373f837334b19b6033c93867fb467.pdf)
        * <http://archive.today/2026.03.18-032824/https://www.pomoculture.org/2020/06/25/how-natives-drink-bravo-shots-for-example-mourning-and-nuclear-kitsch/>
        * [Constitution Day](https://web.archive.org/web/20241114230942/https://www.pscrmi.net/_files/ugd/8c401f_048d743d0fd742df980a09fce125c8fa.pdf)
        * [Fisherman's Day](https://web.archive.org/web/20241115013042/https://www.pscrmi.net/_files/ugd/8c401f_c1448f1df14a4244bb6391179aa85e5c.pdf)
        * [Manit Day](https://web.archive.org/web/20241115012706/https://www.pscrmi.net/_files/ugd/8c401f_39a1a9974cf54827b2017acc73a5018d.pdf)
        * [2019-2026](https://web.archive.org/web/20260208042153/https://www.pscrmi.net/rmi-holiday-memos)
    """

    country = "MH"
    # %s (observed).
    observed_label = "%s Holiday"
    # Public Holiday Act 1988.
    start_year = 1989

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_TO_PREV_FRI + SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = "New Year's Day"
        self._add_observed(self._add_new_years_day(name))
        self._add_observed(self._next_year_new_years_day, name=name)

        # Public Law 1995-134.
        # P.L. 2003-99 (Dec 2003): renamed to Memorial Day and Nuclear Survivors Remembrance Day.
        # P.L. 2005-35 (~May 2005): reverted to Nuclear Victims Remembrance Day.
        if self._year >= 1996:
            self._add_observed(
                self._add_holiday_mar_1(
                    # Memorial Day and Nuclear Survivors Remembrance Day.
                    "Memorial Day and Nuclear Survivors Remembrance Day"
                    if 2004 <= self._year <= 2005
                    # Nuclear Victims Remembrance Day.
                    else "Nuclear Victims Remembrance Day"
                )
            )

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Public Law 2005-35.
        if self._year >= 2006:
            # Constitution Day.
            self._add_observed(self._add_holiday_may_1("Constitution Day"))

        # Public Law 1995-134.
        if self._year >= 1996:
            # Fisherman's Day.
            self._add_holiday_1st_fri_of_jul("Fisherman's Day")

            # Dri-jerbal Day.
            self._add_holiday_1st_fri_of_sep("Dri-jerbal Day")

            # Manit Day.
            self._add_holiday_last_fri_of_sep("Manit Day")

        # President's Day.
        self._add_observed(self._add_holiday_nov_17("President's Day"))

        # Public Law 2015-38.
        if self._year >= 2015 and self._year % 4 == 3:
            # General Election Day.
            self._add_holiday_3rd_mon_of_nov("General Election Day")

        # Gospel Day.
        self._add_holiday_1st_fri_of_dec("Gospel Day")

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"))


class HolidaysMH(MarshallIslands):
    pass


class MH(MarshallIslands):
    pass


class MHL(MarshallIslands):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.norfolk_island import NorfolkIsland
from tests.common import CommonCountryTests


class TestNorfolkIsland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2016, 2050)
        super().setUpClass(NorfolkIsland, years=years, years_non_observed=years)

    def test_special_holidays(self):
        self.assertHoliday("2022-09-22")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2016, 2050)))
        obs_dt = (
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_australia_day(self):
        name = "Australia Day"
        self.assertHolidayName(name, (f"{year}-01-26" for year in range(2016, 2050)))
        obs_dt = (
            "2019-01-28",
            "2020-01-27",
            "2025-01-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_foundation_day(self):
        name = "Foundation Day"
        self.assertHolidayName(name, (f"{year}-03-06" for year in range(2016, 2050)))
        obs_dt = (
            "2016-03-07",
            "2022-03-07",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2016, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(2016, 2050))

    def test_anzac_day(self):
        self.assertHolidayName("ANZAC Day", (f"{year}-04-25" for year in range(2016, 2050)))

    def test_bounty_day(self):
        name = "Bounty Day"
        self.assertHolidayName(name, (f"{year}-06-08" for year in range(2016, 2050)))
        obs_dt = (
            "2019-06-10",
            "2024-06-10",
            "2025-06-09",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_sovereigns_birthday(self):
        name_queen = "Queen's Birthday"
        self.assertHolidayName(
            name_queen,
            "2016-06-13",
            "2017-06-12",
            "2018-06-11",
            "2019-06-17",
            "2020-06-15",
            "2021-06-14",
            "2022-06-13",
        )
        self.assertNoHolidayName(name_queen, range(2023, 2050))

        name_king = "King's Birthday"
        self.assertHolidayName(
            name_king,
            "2023-06-12",
            "2024-06-17",
            "2025-06-16",
        )
        self.assertHolidayName(name_king, range(2023, 2050))
        self.assertNoHolidayName(name_king, range(2016, 2023))

    def test_show_day(self):
        name = "Show Day"
        self.assertHolidayName(
            name,
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(name, range(2016, 2050))

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        self.assertHolidayName(
            name,
            "2020-11-25",
            "2021-11-24",
            "2022-11-30",
            "2023-11-29",
            "2024-11-27",
            "2025-11-26",
        )
        self.assertHolidayName(name, range(2016, 2050))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2016, 2050)))
        obs_dt = (
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(2016, 2050)))
        obs_dt = (
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-26", "Australia Day"),
            ("2022-03-06", "Foundation Day"),
            ("2022-03-07", "Foundation Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "ANZAC Day"),
            ("2022-06-08", "Bounty Day"),
            ("2022-06-13", "Queen's Birthday"),
            ("2022-09-22", "National Day of Mourning for Queen Elizabeth II"),
            ("2022-10-10", "Show Day"),
            ("2022-11-30", "Thanksgiving Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-26", "Australia Day"),
            ("2022-03-06", "Foundation Day"),
            ("2022-03-07", "Foundation Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "ANZAC Day"),
            ("2022-06-08", "Bounty Day"),
            ("2022-06-13", "Queen's Birthday"),
            ("2022-09-22", "National Day of Mourning for Queen Elizabeth II"),
            ("2022-10-10", "Show Day"),
            ("2022-11-30", "Thanksgiving Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.sao_tome_and_principe import SaoTomeAndPrincipe
from tests.common import CommonCountryTests


class TestSaoTomeAndPrincipe(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2014, 2050)
        super().setUpClass(SaoTomeAndPrincipe, years=years, years_non_observed=range(2020, 2050))
        cls.subdiv_p_holidays = SaoTomeAndPrincipe(subdiv="P", years=years)
        cls.subdiv_p_holidays_non_observed = SaoTomeAndPrincipe(
            subdiv="P", years=years, observed=False
        )

    def test_new_years_day(self):
        name = "Ano Novo"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2014, 2050)))
        obs_dt = ("2021-12-31", "2023-01-02")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_day_of_king_amador(self):
        name = "Dia do Rei Amador"
        self.assertHolidayName(name, (f"{year}-01-04" for year in range(2014, 2050)))
        obs_dt = ("2020-01-03", "2025-01-03")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_martyrs_day(self):
        name = "Dia dos Mártires"
        self.assertHolidayName(name, (f"{year}-02-03" for year in range(2014, 2050)))
        obs_dt = ("2024-02-02",)
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_labor_day(self):
        name = "Dia do Trabalhador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(2014, 2050)))
        obs_dt = ("2021-04-30", "2022-05-02")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_independence_day(self):
        name = "Dia da Independência"
        self.assertHolidayName(name, (f"{year}-07-12" for year in range(2014, 2050)))
        obs_dt = ("2020-07-13", "2025-07-11")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_armed_forces_day(self):
        name = "Dia das Forças Armadas"
        self.assertHolidayName(name, (f"{year}-09-06" for year in range(2014, 2050)))
        obs_dt = ("2020-09-07", "2025-09-05")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_agrarian_reform_day(self):
        name = "Dia da Reforma Agrária"
        self.assertHolidayName(name, (f"{year}-09-30" for year in range(2014, 2050)))
        obs_dt = ("2023-09-29",)
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_sao_tome_day(self):
        name = "Dia de São Tomé"
        self.assertHolidayName(name, (f"{year}-12-21" for year in range(2019, 2050)))
        self.assertNoHolidayName(name, range(2014, 2019))
        obs_dt = ("2024-12-20", "2025-12-22")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        name = "Natal"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2014, 2050)))
        obs_dt = ("2021-12-24", "2022-12-26")
        self.assertHolidayName(f"{name} (observado)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_discovery_of_principe_island(self):
        name = "Descobrimento da Ilha do Príncipe"
        self.assertHolidayName(
            name, self.subdiv_p_holidays, (f"{year}-01-17" for year in range(2014, 2050))
        )
        self.assertNoHolidayName(name)
        obs_dt = ("2021-01-18",)
        self.assertHolidayName(f"{name} (observado)", self.subdiv_p_holidays, obs_dt)
        self.assertNoNonObservedHoliday(self.subdiv_p_holidays_non_observed, obs_dt)

    def test_autonomy_day(self):
        name = "Dia da Autonomia do Príncipe"
        self.assertHolidayName(
            name, self.subdiv_p_holidays, (f"{year}-04-29" for year in range(2014, 2050))
        )
        self.assertNoHolidayName(name)
        obs_dt = ("2023-04-28",)
        self.assertHolidayName(f"{name} (observado)", self.subdiv_p_holidays, obs_dt)
        self.assertNoNonObservedHoliday(self.subdiv_p_holidays_non_observed, obs_dt)

    def test_sao_lorenco_day(self):
        name = "Dia de São Lourenço"
        self.assertHolidayName(
            name, self.subdiv_p_holidays, (f"{year}-08-15" for year in range(2014, 2050))
        )
        self.assertNoHolidayName(name)
        obs_dt = ("2020-08-14", "2021-08-16")
        self.assertHolidayName(f"{name} (observado)", self.subdiv_p_holidays, obs_dt)
        self.assertNoNonObservedHoliday(self.subdiv_p_holidays_non_observed, obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Ano Novo"),
            ("2022-01-04", "Dia do Rei Amador"),
            ("2022-01-17", "Descobrimento da Ilha do Príncipe"),
            ("2022-02-03", "Dia dos Mártires"),
            ("2022-04-29", "Dia da Autonomia do Príncipe"),
            ("2022-05-01", "Dia do Trabalhador"),
            ("2022-05-02", "Dia do Trabalhador (observado)"),
            ("2022-07-12", "Dia da Independência"),
            ("2022-08-15", "Dia de São Lourenço"),
            ("2022-09-06", "Dia das Forças Armadas"),
            ("2022-09-30", "Dia da Reforma Agrária"),
            ("2022-12-21", "Dia de São Tomé"),
            ("2022-12-25", "Natal"),
            ("2022-12-26", "Natal (observado)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-04", "Day of King Amador"),
            ("2022-01-17", "Discovery of Príncipe Island"),
            ("2022-02-03", "Martyrs' Day"),
            ("2022-04-29", "Autonomy Day"),
            ("2022-05-01", "Worker's Day"),
            ("2022-05-02", "Worker's Day (observed)"),
            ("2022-07-12", "Independence Day"),
            ("2022-08-15", "São Lourenço Day"),
            ("2022-09-06", "Armed Forces Day"),
            ("2022-09-30", "Agricultural Reform Day"),
            ("2022-12-21", "São Tomé Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import _timedelta
from holidays.constants import PUBLIC, UNOFFICIAL, WORKDAY
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Finland(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Finland holidays.

    Official Flag Days are included in the `WORKDAY` category, while Customary Flag Days
    are included in `UNOFFICIAL` category.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Finland>
        * [Bank holidays (Finnish)](https://web.archive.org/web/20250416185850/https://www.suomenpankki.fi/fi/raha-ja-maksaminen/pankkivapaapaivat/)
        * [Bank holidays (English)](https://web.archive.org/web/20250327200736/https://www.suomenpankki.fi/en/money-and-payments/bank-holidays/)
        * [Bank holidays (Swedish)](https://web.archive.org/web/20250217014536/https://www.suomenpankki.fi/sv/pengar-och-betalningar/bankfria-dagar-i-finland/)
        * <https://en.wikipedia.org/wiki/Flag_flying_days_in_Finland#Customary_flag_days>
        * <https://web.archive.org/web/20250327201901/https://intermin.fi/en/flag-and-arms/flag-flying-days>
        * <https://web.archive.org/web/20241227163324/https://intermin.fi/en/flag-and-arms/flag-days/2024>
        * <https://web.archive.org/web/20250515053310/https://almanakka.helsinki.fi/en/flag-days-and-holidays/>
        * <https://web.archive.org/web/20250515054129/https://www.finlex.fi/en/legislation/collection/1978/383>
        * <https://web.archive.org/web/20250515064749/https://itsenäisyys.fi/itsenaisyyspaivan-perinteet-vakiintuivat-jo-itsenaisyyden-alkuvuosina/>
        * <https://web.archive.org/web/20250515072453/https://yle.fi/a/20-152398>
        * <https://en.wikipedia.org/wiki/Independence_Day_(Finland)>
        * <https://en.wikipedia.org/wiki/Åland's_Autonomy_Day>
        * <https://web.archive.org/web/20250509184304/https://wiki.aineetonkulttuuriperinto.fi/wiki/Workers’_Labour_Day_on_May_1>
    """

    country = "FI"
    default_language = "fi"
    supported_languages = ("en_US", "fi", "sv_FI", "th", "uk")
    supported_categories = (PUBLIC, UNOFFICIAL, WORKDAY)
    subdivisions: tuple[str, ...] = (
        "01",  # Ahvenanmaan maakunta (Landskapet Åland).
        "02",  # Etelä-Karjala (Södra Karelen).
        "03",  # Etelä-Pohjanmaa (Södra Österbotten).
        "04",  # Etelä-Savo (Södra Savolax).
        "05",  # Kainuu (Kajanaland).
        "06",  # Kanta-Häme (Egentliga Tavastland).
        "07",  # Keski-Pohjanmaa (Mellersta Österbotten).
        "08",  # Keski-Suomi (Mellersta Finland).
        "09",  # Kymenlaakso (Kymmenedalen).
        "10",  # Lappi (Lappland).
        "11",  # Pirkanmaa (Birkaland).
        "12",  # Pohjanmaa (Österbotten).
        "13",  # Pohjois-Karjala (Norra Karelen).
        "14",  # Pohjois-Pohjanmaa (Norra Österbotten).
        "15",  # Pohjois-Savo (Norra Savolax).
        "16",  # Päijät-Häme (Päijänne-Tavastland).
        "17",  # Satakunta.
        "18",  # Uusimaa (Nyland).
        "19",  # Varsinais-Suomi (Egentliga Finland).
    )
    subdivisions_aliases = {
        "Ahvenanmaan maakunta": "01",
        "Landskapet Åland": "01",
        "Etelä-Karjala": "02",
        "Södra Karelen": "02",
        "Etelä-Pohjanmaa": "03",
        "Södra Österbotten": "03",
        "Etelä-Savo": "04",
        "Södra Savolax": "04",
        "Kainuu": "05",
        "Kajanaland": "05",
        "Kanta-Häme": "06",
        "Egentliga Tavastland": "06",
        "Keski-Pohjanmaa": "07",
        "Mellersta Österbotten": "07",
        "Keski-Suomi": "08",
        "Mellersta Finland": "08",
        "Kymenlaakso": "09",
        "Kymmenedalen": "09",
        "Lappi": "10",
        "Lappland": "10",
        "Pirkanmaa": "11",
        "Birkaland": "11",
        "Pohjanmaa": "12",
        "Österbotten": "12",
        "Pohjois-Karjala": "13",
        "Norra Karelen": "13",
        "Pohjois-Pohjanmaa": "14",
        "Norra Österbotten": "14",
        "Pohjois-Savo": "15",
        "Norra Savolax": "15",
        "Päijät-Häme": "16",
        "Päijänne-Tavastland": "16",
        "Satakunta": "17",
        "Uusimaa": "18",
        "Nyland": "18",
        "Varsinais-Suomi": "19",
        "Egentliga Finland": "19",
    }
    start_year = 1853

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Uudenvuodenpäivä"))

        # Epiphany.
        name = tr("Loppiainen")
        if 1973 <= self._year <= 1990:
            self._add_holiday_1st_sat_from_jan_6(name)
        else:
            self._add_epiphany_day(name)

        # Good Friday.
        self._add_good_friday(tr("Pitkäperjantai"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Pääsiäispäivä"))

        # Easter Monday.
        self._add_easter_monday(tr("Toinen pääsiäispäivä"))

        if self._year >= 1944:
            # May Day.
            self._add_holiday_may_1(tr("Vappu"))

        # Ascension Day.
        name = tr("Helatorstai")
        if 1973 <= self._year <= 1990:
            self._add_holiday_34_days_past_easter(name)
        else:
            self._add_ascension_thursday(name)

        # Whit Sunday.
        self._add_whit_sunday(tr("Helluntaipäivä"))

        # Midsummer Eve.
        name = tr("Juhannusaatto")
        if self._year >= 1955:
            dt = self._add_holiday_1st_fri_from_jun_19(name)
        else:
            dt = self._add_holiday_jun_23(name)

        # Midsummer Day.
        self._add_holiday(tr("Juhannuspäivä"), _timedelta(dt, +1))

        # All Saints' Day.
        name = tr("Pyhäinpäivä")
        if self._year >= 1955:
            self._add_holiday_1st_sat_from_oct_31(name)
        else:
            self._add_holiday_nov_1(name)

        # Designated as Public Holiday on NOV 20th, 1919.
        if self._year >= 1919:
            # Independence Day.
            self._add_holiday_dec_6(tr("Itsenäisyyspäivä"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Jouluaatto"))

        # Christmas Day.
        self._add_christmas_day(tr("Joulupäivä"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Tapaninpäivä"))

    def _populate_unofficial_holidays(self):
        # Customary Flag Days.

        # Unofficial observance starts in 1854.
        # Starting in 1929, become a name day of J.L. Runeberg.
        # Added to the Almanac in 1950 as J.L. Runeberg's Day.
        # Become a Flag Day in 1976.
        if self._year >= 1976:
            # Runeberg Day.
            self._add_holiday_feb_5(tr("Runebergin päivä"))

        # Petition for Flag Day status starts in 2003.
        # Become a Flag Day in 2007.
        if self._year >= 2007:
            #  Minna Canth Day, Day of Equality.
            self._add_holiday_mar_19(tr("Minna Canthin päivä, tasa-arvon päivä"))

        # Added to the Almanac in 1960 as Mikael Agricola Day.
        # Also considered the "Day of Finnish Language" from 1980 onwards.
        # Become a Flag Day in 1980.
        if self._year >= 1980:
            # Mikael Agricola Day, Day of the Finnish Language.
            self._add_holiday_apr_9(tr("Mikael Agricolan päivä, suomen kielen päivä"))

        # Become a Flag Day in 1987.
        if self._year >= 1987:
            # National War Veterans' Day.
            self._add_holiday_apr_27(tr("Kansallinen veteraanipäivä"))

        # Become a Flag Day in 2019.
        if self._year >= 2019:
            # Europe Day.
            self._add_europe_day(tr("Eurooppa-päivä"))

        # Petition for Flag Day status starts in the 1920s.
        # Become a Flag Day in 1952.
        # Also considered the "Day of Finnish Heritage" from 1978 onward.
        if self._year >= 1952:
            self._add_holiday_may_12(
                # J. V. Snellman Day, Day of Finnish Heritage.
                tr("J.V. Snellmanin päivä, suomalaisuuden päivä")
                if self._year >= 1978
                # J. V. Snellman Day.
                else tr("J.V. Snellmanin päivä")
            )

        # Become a Flag Day in 1977.
        if self._year >= 1977:
            # Remembrance Day.
            self._add_holiday_3rd_sun_of_may(tr("Kaatuneitten muistopäivä"))

        # Become a Flag Day in 1998.
        if self._year >= 1998:
            # Eino Leino Day, Day of Summer and Poetry.
            self._add_holiday_jul_6(tr("Eino Leinon päivä, runon ja suven päivä"))

        # Added to the Almanac in 2020.
        # Become a Flag Day in 2023.
        if self._year >= 2023:
            # Finland's Nature Day.
            self._add_holiday_last_sat_of_aug(tr("Suomen luonnon päivä"))

        # Become a Flag Day in 2016.
        if self._year >= 2016:
            # Miina Sillanpää Day, Day of Civic Participation.
            self._add_holiday_oct_1(tr("Miina Sillanpään ja kansalaisvaikuttamisen päivä"))

        # Become a Flag Day in 1950.
        # Also considered the "Day of Finnish Literature" from 1978 onward.
        if self._year >= 1950:
            self._add_holiday_oct_10(
                # Aleksis Kivi Day, Day of Finnish Literature.
                tr("Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä")
                if self._year >= 1978
                # Aleksis Kivi Day.
                else tr("Aleksis Kiven päivä")
            )

        # Become a Flag Day in 1987.
        if self._year >= 1987:
            # United Nations Day.
            self._add_united_nations_day(tr("YK:n päivä"))

        # Become a Flag Day in 1979.
        if self._year >= 1979:
            # Finnish Swedish Heritage Day, svenska dagen.
            self._add_holiday_nov_6(tr("Ruotsalaisuuden päivä, Kustaa Aadolfin päivä"))

        # Become a Customary Flag Day in 1987.
        # Become an Official Flag Day in 2019.
        if 1987 <= self._year <= 2018:
            # Father's Day.
            self._add_holiday_2nd_sun_of_nov(tr("Isänpäivä"))

        # Become a Flag Day in 2020.
        if self._year >= 2020:
            # Day of Children's Rights.
            self._add_holiday_nov_20(tr("Lapsen oikeuksien päivä"))

        # First recommendation starts in 2005.
        # Become a Flag Day in 2011.
        if self._year >= 2011:
            # Jean Sibelius Day, Day of Finnish Music.
            self._add_holiday_dec_8(tr("Jean Sibeliuksen päivä, suomalaisen musiikin päivä"))

    def _populate_workday_holidays(self):
        # Official Flag Days.

        # First general observance starts in 1885.
        # Become a Flag Day in 1920.
        # Added to the Almanac in 1950, confirmed by a decree in 1978.
        if self._year >= 1920:
            # Kalevala Day, Day of Finnish Culture.
            self._add_holiday_feb_28(tr("Kalevalan päivä, suomalaisen kulttuurin päivä"))

        # While May Day is already a Public Holiday since 1944, it gains Flag Day status in 1978,
        # although actual observance starts in 1979.
        if self._year >= 1979:
            # May Day.
            self._add_holiday_may_1(tr("Vappu"))

        # First observed in parts of Finland in 1918.
        # Moved from 3rd to 2nd Sunday of May in 1927.
        # Become Flag Day in 1947.
        if self._year >= 1947:
            # Mother's Day.
            self._add_holiday_2nd_sun_of_may(tr("Äitienpäivä"))

        # Become a Flag Day in 1942.
        # Got its current name in 1950.
        if self._year >= 1942:
            self._add_holiday_jun_6(
                # Flag Day of the Finnish Defense Forces.
                tr("Puolustusvoimain lippujuhlan päivä")
                if self._year >= 1950
                # Birthday of the Marshal of Finland.
                else tr("Suomen marsalkan syntymäpäivä")
            )

        # Day of the Finnish Flag was first created in 1934.
        # This coincides with Midsummer Day.

        # Day of the Finnish Flag.
        name = tr("Suomen lipun päivä")
        if self._year >= 1955:
            self._add_holiday_1st_sat_from_jun_20(name)
        elif self._year >= 1934:
            self._add_holiday_jun_24(name)

        # Become a Customary Flag Day in 1987.
        # Become an Official Flag Day in 2019.
        if self._year >= 2019:
            # Father's Day.
            self._add_holiday_2nd_sun_of_nov(tr("Isänpäivä"))

        # Flag Day status assumed to start in 1919.
        if self._year >= 1919:
            # Independence Day.
            self._add_holiday_dec_6(tr("Itsenäisyyspäivä"))

    def _populate_subdiv_01_public_holidays(self):
        # Celebrated since 1993 when the 70th anniversary of the Autonomy Act of 1920 was
        # first formally recognized as a flag day.
        if self._year >= 1993:
            # Åland's Autonomy Day.
            self._add_holiday_jun_9(tr("Ahvenanmaan itsehallintopäivä"))


class FI(Finland):
    pass


class FIN(Finland):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.pitcairn_islands import PitcairnIslands
from tests.common import CommonCountryTests


class TestPitcairnIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(PitcairnIslands)

    def test_new_years_day(self):
        name = "New Year's Day"
        # Public Holidays.
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertNoHolidayName(name, (f"{year}-01-02" for year in self.full_range))
        # Government Holidays.
        self.assertGovernmentHolidayName(name, (f"{year}-01-02" for year in self.full_range))

    def test_bounty_day(self):
        self.assertHolidayName("Bounty Day", (f"{year}-01-23" for year in self.full_range))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "2016-06-11",
            "2017-06-10",
            "2018-06-09",
            "2019-06-08",
            "2020-06-13",
            "2021-06-12",
            "2022-06-11",
        )
        self.assertNoHolidayName(name, range(2023, self.end_year))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            "2023-06-05",
            "2024-06-03",
            "2025-06-02",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_pitcairn_day(self):
        self.assertHolidayName("Pitcairn Day", (f"{year}-07-02" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in self.full_range))

    def test_anzac_day(self):
        name = "ANZAC Day"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-04-25" for year in self.full_range))

    def test_remembrance_day(self):
        name = "Remembrance Day"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-11-11" for year in self.full_range))

    def test_2022_public(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-23", "Bounty Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-06-11", "Queen's Birthday"),
            ("2022-07-02", "Pitcairn Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )

    def test_2022_government(self):
        self.assertGovernmentHolidaysInYear(
            2022,
            ("2022-01-02", "New Year's Day"),
        )

    def test_2022_workday(self):
        self.assertWorkdayHolidaysInYear(
            2022,
            ("2022-04-25", "ANZAC Day"),
            ("2022-11-11", "Remembrance Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.azerbaijan import Azerbaijan
from tests.common import CommonCountryTests, WorkingDayTests


class TestAzerbaijan(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(1990, 2073)
        super().setUpClass(Azerbaijan)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Azerbaijan(categories=WORKDAY, years=range(self.start_year, 1992)))

    def test_special_holidays(self):
        self.assertHoliday(
            "2007-01-03",
            "2018-04-11",
            "2019-12-27",
            "2024-02-07",
            "2025-01-29",
            "2025-03-27",
            "2025-03-28",
            "2072-01-05",
        )

    def test_substituted_holidays(self):
        self.assertHoliday(
            "2011-08-29",
            "2013-01-03",
            "2013-01-04",
            "2014-01-03",
            "2014-01-06",
            "2020-01-03",
            "2020-01-06",
            "2020-03-27",
            "2020-05-27",
            "2021-05-11",
            "2021-05-12",
            "2021-07-19",
            "2022-03-07",
            "2022-11-07",
            "2023-06-27",
            "2023-06-30",
            "2023-11-10",
            "2024-01-04",
            "2024-01-05",
            "2024-04-12",
            "2024-11-12",
            "2024-11-13",
            "2024-12-30",
            "2025-01-03",
            "2025-06-27",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "2011-08-27",
            "2012-12-29",
            "2012-12-30",
            "2013-12-28",
            "2013-12-29",
            "2019-12-28",
            "2019-12-29",
            "2020-03-29",
            "2020-05-30",
            "2021-05-08",
            "2021-05-16",
            "2021-07-17",
            "2022-03-05",
            "2022-11-05",
            "2023-06-24",
            "2023-06-25",
            "2023-11-04",
            "2023-12-30",
            "2024-01-07",
            "2024-04-06",
            "2024-11-16",
            "2024-11-23",
            "2024-12-28",
            "2024-12-29",
            "2025-06-21",
        )

        for year, dts in {
            2012: (
                "2012-12-29",
                "2012-12-30",
            ),
            2013: (
                "2013-12-28",
                "2013-12-29",
            ),
            2019: (
                "2019-12-28",
                "2019-12-29",
            ),
            2023: ("2023-12-30",),
            2024: ("2024-12-29",),
        }.items():
            self.assertWorkingDay(Azerbaijan(years=year), dts)

    def test_new_years_day(self):
        name = "Yeni il bayramı"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertHolidayName(name, (f"{year}-01-02" for year in range(2006, self.end_year)))
        self.assertNoHolidayName(name, (f"{year}-01-02" for year in range(self.start_year, 2006)))
        obs_dts = (
            "2021-01-04",
            "2022-01-03",
            "2022-01-04",
            "2023-01-04",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_martyrs_day(self):
        name = "Ümumxalq hüzn günü"
        self.assertHolidayName(name, (f"{year}-01-20" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2000))

    def test_womens_day(self):
        name = "Qadınlar günü"
        self.assertHolidayName(name, (f"{year}-03-08" for year in self.full_range))
        obs_dts = (
            "2014-03-10",
            "2015-03-09",
            "2020-03-09",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_spring_festival(self):
        name = "Novruz bayramı"
        self.assertHolidayName(name, (f"{year}-03-20" for year in self.full_range))
        self.assertHolidayName(name, (f"{year}-03-21" for year in self.full_range))
        self.assertHolidayName(name, (f"{year}-03-22" for year in range(2007, self.end_year)))
        self.assertHolidayName(name, (f"{year}-03-23" for year in range(2007, self.end_year)))
        self.assertHolidayName(name, (f"{year}-03-24" for year in range(2007, self.end_year)))
        obs_dts = (
            "2024-03-25",
            "2024-03-26",
            "2025-03-25",
            "2025-03-26",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_victory_over_fascism_day(self):
        name = "Faşizm üzərində qələbə günü"
        self.assertHolidayName(name, (f"{year}-05-09" for year in self.full_range))
        obs_dts = (
            "2010-05-10",
            "2015-05-11",
            "2020-05-11",
            "2021-05-10",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_republic_day(self):
        name_1992 = "Respublika Günü"
        name_2021 = "Müstəqillik Günü"
        self.assertHolidayName(name_1992, (f"{year}-05-28" for year in range(1992, 2021)))
        self.assertHolidayName(name_2021, (f"{year}-05-28" for year in range(2021, self.end_year)))
        self.assertNoHolidayName(
            name_1992, range(self.start_year, 1992), range(2021, self.end_year)
        )
        self.assertNoHolidayName(name_2021, range(self.start_year, 2021))
        obs_dts_1992 = (
            "2006-05-29",
            "2011-05-30",
            "2016-05-30",
            "2017-05-29",
        )
        self.assertHolidayName(f"{name_1992} (müşahidə olunur)", obs_dts_1992)
        obs_dts_2021 = (
            "2022-05-30",
            "2023-05-29",
        )
        self.assertHolidayName(f"{name_2021} (müşahidə olunur)", obs_dts_2021)
        self.assertNoNonObservedHoliday(obs_dts_1992, obs_dts_2021)

    def test_national_liberation_day(self):
        name = "Azərbaycan xalqının milli qurtuluş günü"
        self.assertHolidayName(name, (f"{year}-06-15" for year in range(1997, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1997))
        obs_dts = (
            "2013-06-17",
            "2014-06-16",
            "2019-06-17",
            "2024-06-18",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_armed_forces_day(self):
        name = "Azərbaycan Respublikasının Silahlı Qüvvələri günü"
        self.assertHolidayName(name, (f"{year}-10-09" for year in range(1992, 1998)))
        self.assertHolidayName(name, (f"{year}-06-26" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1992))
        obs_dts = (
            "2011-06-27",
            "2016-06-27",
            "2021-06-28",
            "2022-06-27",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_memorial_day(self):
        name = "Anım Günü"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-09-27" for year in range(2021, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2021))

    def test_independence_restoration_day(self):
        name_1992 = "Milli Müstəqillik Günü"
        name_2021 = "Müstəqilliyin Bərpası Günü"
        self.assertHolidayName(
            name_1992, (f"{year}-10-18" for year in range(self.start_year, 2006))
        )
        self.assertNoHolidayName(name_1992, range(2006, self.end_year))

        self.assertWorkdayHolidayName(name_1992, (f"{year}-10-18" for year in range(2006, 2021)))
        self.assertWorkdayHolidayName(
            name_2021, (f"{year}-10-18" for year in range(2021, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name_1992, range(2021, self.end_year))
        self.assertNoWorkdayHolidayName(name_2021, range(self.start_year, 2021))

    def test_victory_day(self):
        name = "Zəfər Günü"
        self.assertHolidayName(name, (f"{year}-11-08" for year in range(2021, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2021))
        obs_dts = ("2025-11-10",)
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_flag_day(self):
        name = "Azərbaycan Respublikasının Dövlət bayrağı günü"
        self.assertHolidayName(name, (f"{year}-11-09" for year in range(2010, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2010))
        obs_dts = (
            "2014-11-10",
            "2019-11-11",
            "2024-11-11",
            "2025-11-11",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name = "Konstitusiya Günü"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-11-12" for year in range(1996, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1996))

    def test_revival_day(self):
        name = "Milli Dirçəliş Günü"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-11-17" for year in range(1992, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1992))

    def test_international_azerbaijanis_solidarity_day(self):
        name = "Dünya azərbaycanlılarının həmrəyliyi günü"
        self.assertHolidayName(name, (f"{year}-12-31" for year in range(1993, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1993))
        obs_dts = (
            "2017-01-03",
            "2018-01-03",
            "2023-01-03",
            "2024-01-03",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Ramazan bayrami"
        self.assertHolidayName(
            name,
            "2005-11-03",
            "2006-10-23",
            "2006-10-24",
            "2020-05-24",
            "2020-05-25",
            "2021-05-13",
            "2021-05-14",
            "2022-05-02",
            "2022-05-03",
            "2023-04-21",
            "2023-04-22",
        )
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 1, range(1993, 2000), range(2001, 2006)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 2, 2000, range(2006, 2033), range(2034, 2065), range(2066, self.end_year)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 2033, 2065)
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1993))
        obs_dts = (
            "2018-06-18",
            "2018-06-19",
            "2020-05-26",
            "2023-04-24",
        )
        self.assertHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Qurban bayrami"
        self.assertHolidayName(
            name,
            "2006-01-10",
            "2006-12-31",
            "2020-07-31",
            "2020-08-01",
            "2021-07-20",
            "2021-07-21",
            "2022-07-09",
            "2022-07-10",
            "2023-06-28",
            "2023-06-29",
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 1, range(1993, 2006))
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 2, 2006, range(2008, 2039), range(2040, 2071)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 3, 2007, range(2071, self.end_year))
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 2039)
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1993))
        obs_dts = (
            "2022-07-11",
            "2022-07-12",
            "2024-06-19",
            "2025-06-09",
            # Special Cases.
            "2007-01-03",
            "2072-01-05",
        )
        self.assertIslamicNoEstimatedHolidayName(f"{name} (müşahidə olunur)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2021(self):
        self.assertHolidayDatesInYear(
            2021,
            "2021-01-01",
            "2021-01-02",
            "2021-01-04",
            "2021-01-20",
            "2021-03-08",
            "2021-03-20",
            "2021-03-21",
            "2021-03-22",
            "2021-03-23",
            "2021-03-24",
            "2021-03-25",
            "2021-03-26",
            "2021-05-09",
            "2021-05-10",
            "2021-05-11",
            "2021-05-12",
            "2021-05-13",
            "2021-05-14",
            "2021-05-28",
            "2021-06-15",
            "2021-06-26",
            "2021-06-28",
            "2021-07-19",
            "2021-07-20",
            "2021-07-21",
            "2021-11-08",
            "2021-11-09",
            "2021-12-31",
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Yeni il bayramı"),
            ("2022-01-02", "Yeni il bayramı"),
            ("2022-01-03", "Yeni il bayramı (müşahidə olunur)"),
            ("2022-01-04", "Yeni il bayramı (müşahidə olunur)"),
            ("2022-01-20", "Ümumxalq hüzn günü"),
            ("2022-03-07", "İstirahət günü (05.03.2022 ilə əvəz edilmişdir)"),
            ("2022-03-08", "Qadınlar günü"),
            ("2022-03-20", "Novruz bayramı"),
            ("2022-03-21", "Novruz bayramı"),
            ("2022-03-22", "Novruz bayramı"),
            ("2022-03-23", "Novruz bayramı"),
            ("2022-03-24", "Novruz bayramı"),
            ("2022-03-25", "Novruz bayramı (müşahidə olunur)"),
            ("2022-05-02", "Ramazan bayrami"),
            ("2022-05-03", "Ramazan bayrami"),
            ("2022-05-09", "Faşizm üzərində qələbə günü"),
            ("2022-05-28", "Müstəqillik Günü"),
            ("2022-05-30", "Müstəqillik Günü (müşahidə olunur)"),
            ("2022-06-15", "Azərbaycan xalqının milli qurtuluş günü"),
            ("2022-06-26", "Azərbaycan Respublikasının Silahlı Qüvvələri günü"),
            ("2022-06-27", "Azərbaycan Respublikasının Silahlı Qüvvələri günü (müşahidə olunur)"),
            ("2022-07-09", "Qurban bayrami"),
            ("2022-07-10", "Qurban bayrami"),
            ("2022-07-11", "Qurban bayrami (müşahidə olunur)"),
            ("2022-07-12", "Qurban bayrami (müşahidə olunur)"),
            ("2022-11-07", "İstirahət günü (05.11.2022 ilə əvəz edilmişdir)"),
            ("2022-11-08", "Zəfər Günü"),
            ("2022-11-09", "Azərbaycan Respublikasının Dövlət bayrağı günü"),
            ("2022-12-31", "Dünya azərbaycanlılarının həmrəyliyi günü"),
        )

    def test_2022_workday(self):
        self.assertWorkdayHolidaysInYear(
            2022,
            ("2022-09-27", "Anım Günü"),
            ("2022-10-18", "Müstəqilliyin Bərpası Günü"),
            ("2022-11-12", "Konstitusiya Günü"),
            ("2022-11-17", "Milli Dirçəliş Günü"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Yeni il bayramı"),
            ("2023-01-02", "Yeni il bayramı"),
            ("2023-01-03", "Dünya azərbaycanlılarının həmrəyliyi günü (müşahidə olunur)"),
            ("2023-01-04", "Yeni il bayramı (müşahidə olunur)"),
            ("2023-01-20", "Ümumxalq hüzn günü"),
            ("2023-03-08", "Qadınlar günü"),
            ("2023-03-20", "Novruz bayramı"),
            ("2023-03-21", "Novruz bayramı"),
            ("2023-03-22", "Novruz bayramı"),
            ("2023-03-23", "Novruz bayramı"),
            ("2023-03-24", "Novruz bayramı"),
            ("2023-04-21", "Ramazan bayrami"),
            ("2023-04-22", "Ramazan bayrami"),
            ("2023-04-24", "Ramazan bayrami (müşahidə olunur)"),
            ("2023-05-09", "Faşizm üzərində qələbə günü"),
            ("2023-05-28", "Müstəqillik Günü"),
            ("2023-05-29", "Müstəqillik Günü (müşahidə olunur)"),
            ("2023-06-15", "Azərbaycan xalqının milli qurtuluş günü"),
            ("2023-06-26", "Azərbaycan Respublikasının Silahlı Qüvvələri günü"),
            ("2023-06-27", "İstirahət günü (24.06.2023 ilə əvəz edilmişdir)"),
            ("2023-06-28", "Qurban bayrami"),
            ("2023-06-29", "Qurban bayrami"),
            ("2023-06-30", "İstirahət günü (25.06.2023 ilə əvəz edilmişdir)"),
            ("2023-09-27", "Anım Günü"),
            ("2023-10-18", "Müstəqilliyin Bərpası Günü"),
            ("2023-11-08", "Zəfər Günü"),
            ("2023-11-09", "Azərbaycan Respublikasının Dövlət bayrağı günü"),
            ("2023-11-10", "İstirahət günü (04.11.2023 ilə əvəz edilmişdir)"),
            ("2023-11-12", "Konstitusiya Günü"),
            ("2023-11-17", "Milli Dirçəliş Günü"),
            ("2023-12-31", "Dünya azərbaycanlılarının həmrəyliyi günü"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day"),
            ("2023-01-03", "International Azerbaijanis Solidarity Day (observed)"),
            ("2023-01-04", "New Year's Day (observed)"),
            ("2023-01-20", "Martyrs' Day"),
            ("2023-03-08", "Women's Day"),
            ("2023-03-20", "Spring Festival"),
            ("2023-03-21", "Spring Festival"),
            ("2023-03-22", "Spring Festival"),
            ("2023-03-23", "Spring Festival"),
            ("2023-03-24", "Spring Festival"),
            ("2023-04-21", "Eid al-Fitr"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-24", "Eid al-Fitr (observed)"),
            ("2023-05-09", "Victory over Fascism Day"),
            ("2023-05-28", "Independence Day"),
            ("2023-05-29", "Independence Day (observed)"),
            ("2023-06-15", "National Liberation Day"),
            ("2023-06-26", "Armed Forces Day"),
            ("2023-06-27", "Day off (substituted from 06/24/2023)"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-06-30", "Day off (substituted from 06/25/2023)"),
            ("2023-09-27", "Memorial Day"),
            ("2023-10-18", "Independence Restoration Day"),
            ("2023-11-08", "Victory Day"),
            ("2023-11-09", "National Flag Day"),
            ("2023-11-10", "Day off (substituted from 11/04/2023)"),
            ("2023-11-12", "Constitution Day"),
            ("2023-11-17", "National Revival Day"),
            ("2023-12-31", "International Azerbaijanis Solidarity Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-02", "Новий рік"),
            ("2023-01-03", "Всесвітній день солідарності азербайджанців (вихідний)"),
            ("2023-01-04", "Новий рік (вихідний)"),
            ("2023-01-20", "День національної скорботи"),
            ("2023-03-08", "Жіночий день"),
            ("2023-03-20", "Свято Новруз"),
            ("2023-03-21", "Свято Новруз"),
            ("2023-03-22", "Свято Новруз"),
            ("2023-03-23", "Свято Новруз"),
            ("2023-03-24", "Свято Новруз"),
            ("2023-04-21", "Рамазан-байрам"),
            ("2023-04-22", "Рамазан-байрам"),
            ("2023-04-24", "Рамазан-байрам (вихідний)"),
            ("2023-05-09", "День перемоги над фашизмом"),
            ("2023-05-28", "День Незалежності"),
            ("2023-05-29", "День Незалежності (вихідний)"),
            ("2023-06-15", "День національного визволення азербайджанського народу"),
            ("2023-06-26", "День Збройних Сил"),
            ("2023-06-27", "Вихідний день (перенесено з 24.06.2023)"),
            ("2023-06-28", "Курбан-байрам"),
            ("2023-06-29", "Курбан-байрам"),
            ("2023-06-30", "Вихідний день (перенесено з 25.06.2023)"),
            ("2023-09-27", "День памʼяті"),
            ("2023-10-18", "День відновлення незалежності"),
            ("2023-11-08", "День Перемоги"),
            ("2023-11-09", "День державного прапора"),
            ("2023-11-10", "Вихідний день (перенесено з 04.11.2023)"),
            ("2023-11-12", "День Конституції"),
            ("2023-11-17", "День національного відродження"),
            ("2023-12-31", "Всесвітній день солідарності азербайджанців"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import tempfile
from datetime import date, datetime
from pathlib import Path
from unittest import TestCase
from unittest.mock import patch, MagicMock

from holidays import country_holidays, financial_holidays
from holidays.constants import HALF_DAY, UNOFFICIAL
from holidays.holiday_base import HolidayBase
from holidays.ical import CONTENT_LINE_DELIMITER, CONTENT_LINE_MAX_LENGTH, ICalExporter


class MockHolidays(HolidayBase):
    def __init__(self, language: str = "en"):
        super().__init__()
        self.language = language
        self.supported_languages = (language,)

    def _add_custom_holiday(self, name: str, dt: date = date(2024, 1, 1), year: int = 2024):
        super()._populate(year)
        self._add_holiday(name, dt)
        return self


class MockDatetime(datetime):
    @classmethod
    def now(cls, tz=None):
        return cls(2023, 1, 1, 11, 25, 47, tzinfo=tz if tz else None)


class TestIcalExporter(TestCase):
    def setUp(self):
        self.exporter = ICalExporter(None)
        self.exporter.generate = MagicMock(return_value=b"BEGIN:VCALENDAR...END:VCALENDAR")
        self.id_holidays = country_holidays("ID", years=2000, language="en_US")
        self.jp_holidays = country_holidays("JP", years=2000, language="ja")
        self.th_holidays = country_holidays("TH", years=2024, language="th")
        self.us_holidays = country_holidays("US", years=2024, language="en_US")
        self.id_exporter = ICalExporter(self.id_holidays)
        self.jp_exporter = ICalExporter(self.jp_holidays)
        self.th_exporter = ICalExporter(self.th_holidays)
        self.us_exporter = ICalExporter(self.us_holidays)

    def _assert_line_lengths(self, holiday_name):
        test_holidays = MockHolidays()._add_custom_holiday(holiday_name)
        lines = ICalExporter(test_holidays).generate().split(CONTENT_LINE_DELIMITER)
        previous_line = None

        for line in lines:
            self.assertLessEqual(
                len(line.encode()),
                CONTENT_LINE_MAX_LENGTH,
                f"Content line exceeds RFC 5545 content line length: {repr(line)}",
            )

            # If continuation line (starts with a space).
            if line.startswith(" "):
                self.assertIsNot(
                    previous_line,
                    None,
                    f"Continuation line found without a preceding line: {repr(line)}",
                )
            previous_line = line

    def _assert_byte_output(self, exporter, expected_bytes):
        output = exporter.generate(return_bytes=True)
        self.assertIsInstance(output, bytes)
        self.assertIn(expected_bytes, output.split(CONTENT_LINE_DELIMITER.encode()))

    def _assert_special_char_handling(self, escaped_name, sanitized_name):
        test_holidays = MockHolidays()._add_custom_holiday(escaped_name)
        output = ICalExporter(test_holidays).generate()
        self.assertIn(
            f"SUMMARY:{sanitized_name}\r\n",
            output,
            f"Failed for holiday name: {escaped_name}",
        )

    def _assert_valid_language_code(self, language_name):
        language_name = language_name.replace("_", "-")
        test_holidays = MockHolidays(language=language_name)._add_custom_holiday("Test")
        output = ICalExporter(test_holidays, show_language=True).generate()
        self.assertIn(
            f"SUMMARY;LANGUAGE={language_name}:Test\r\n",
            output,
            f"Failed to find holiday with language name: {language_name}",
        )

    def _assert_invalid_language_code(self, language_name):
        language_name = language_name.replace("_", "-")
        test_holidays = MockHolidays(language=language_name)._add_custom_holiday("Test")
        with self.assertRaises(ValueError) as context:
            ICalExporter(test_holidays, show_language=True).generate()
        self.assertEqual(
            str(context.exception),
            (
                f"Invalid language tag: '{language_name}'. Expected format follows "
                "ISO 639-1 or ISO 639-2, e.g., 'en', 'en-US'. For more details, "
                "refer to: https://www.loc.gov/standards/iso639-2/php/code_list.php."
            ),
        )

    def _assert_holidaysum_language(self, holidays, language_name):
        exporter = ICalExporter(holidays, show_language=True)
        lines = exporter.generate().split(CONTENT_LINE_DELIMITER)
        for line in lines:
            if line.startswith("SUMMARY"):
                self.assertIn(
                    f"SUMMARY;LANGUAGE={language_name}:",
                    line,
                    f"Failed to find holiday with language name: {language_name}",
                )

    def _assert_language_not_provided(self, holidays):
        with self.assertRaises(ValueError) as context:
            ICalExporter(holidays, show_language=True)
        self.assertEqual(
            str(context.exception),
            "LANGUAGE cannot be included because the language code is missing.",
        )

    def test_basic_calendar_structure(self):
        output = self.us_exporter.generate()

        # iCalendar File Header.
        self.assertIn("BEGIN:VCALENDAR", output)
        self.assertIn("PRODID:", output)
        self.assertIn("VERSION:2.0", output)
        self.assertIn("CALSCALE:GREGORIAN", output)
        self.assertIn("END:VCALENDAR", output)

        # iCalendar's individual `VEVENT`.
        self.assertIn("BEGIN:VEVENT", output)
        self.assertIn("DTSTAMP:", output)
        self.assertIn("UID:", output)
        self.assertIn("SUMMARY:New Year's Day", output)
        self.assertIn("DTSTART;VALUE=DATE:20240101", output)
        self.assertIn("DURATION:P1D", output)
        self.assertIn("END:VEVENT", output)

    @patch("holidays.ical.datetime", MockDatetime)
    def test_ical_timestamp(self):
        # The whole .ics output should use the same DTSTAMP.
        expected_timestamp = "20230101T112547Z"

        exporter = ICalExporter(self.us_holidays)
        output = exporter.generate()
        lines = output.splitlines()
        for line in lines:
            if line.startswith("DTSTAMP:"):
                self.assertIn(
                    expected_timestamp,
                    line,
                    f"Expected DTSTAMP to contain {expected_timestamp}, but got {line}",
                )

    def test_valid_language_code(self):
        # Primary Code.
        self._assert_valid_language_code("th")
        self._assert_valid_language_code("pap")

        # Primary Code with Region.
        self._assert_valid_language_code("en_US")
        self._assert_valid_language_code("pap_AW")

    def test_invalid_language_code(self):
        # Invalid structures.
        self._assert_invalid_language_code("123-en")
        self._assert_invalid_language_code("en--US")

        # Capitalization.
        self._assert_invalid_language_code("EN-US")
        self._assert_invalid_language_code("en-us")
        self._assert_invalid_language_code("EN-us")

        # Over lang char limit.
        self._assert_invalid_language_code("polynesian")

        # Non-ASCII alphanum.
        self._assert_invalid_language_code("ไทย")

        # Gibberish.
        self._assert_invalid_language_code("abc-!@#")

        # Valid RFC 5646 codes but not ISO 639.
        self._assert_invalid_language_code("i-klingon")
        self._assert_invalid_language_code("es-419")
        self._assert_invalid_language_code("zh_Hant")
        self._assert_invalid_language_code("sl-rozaj")
        self._assert_invalid_language_code("de-CH-1996")
        self._assert_invalid_language_code("az-Latn-AZ")
        self._assert_invalid_language_code("en-US-u-co-emoji")
        self._assert_invalid_language_code("ja-JP-u-ca-japanese")
        self._assert_invalid_language_code("x-mobile")

    def test_show_language_combined_holidays_list(self):
        # HolidaySum init (language provided for both).
        my_holidays = country_holidays("MY", years=2024, language="en_US")
        combined_holidays = my_holidays + self.id_holidays
        self._assert_holidaysum_language(combined_holidays, "en-US")

        # HolidaySum add  (language provided for both).
        sg_holidays = country_holidays("SG", years=2024, language="en_US")
        combined_holidays += sg_holidays
        self._assert_holidaysum_language(combined_holidays, "en-US")

        # HolidaySum (one was not provide language, but default_language exists).
        jp_holidays = country_holidays("JP", years=2000, language="th")
        combined_holidays = self.th_holidays + jp_holidays
        self._assert_holidaysum_language(combined_holidays, "th")

    def test_show_language_true_language_code_none(self):
        # Neither language nor default_language param exists.
        nyse_holidays = financial_holidays("NYSE", years=2024)
        self._assert_language_not_provided(nyse_holidays)

        # Combined Holidays List, one of them is None -> language got set to None.
        self._assert_language_not_provided(nyse_holidays + self.jp_holidays)

        # Combined Holidays List, different language -> language got set to None.
        self._assert_language_not_provided(self.th_holidays + self.jp_holidays)

    def test_multiple_holidays_same_date(self):
        # "วันพ่อแห่งชาติ" and "วันชาติ" are both on DEC 5th.
        output = self.th_exporter.generate()

        self.assertIn("SUMMARY:วันพ่อแห่งชาติ", output)
        self.assertIn("SUMMARY:วันชาติ", output)

    def test_single_holiday_multiple_date_continuous(self):
        # 3x "วันสงกรานต์", 1x "ชดเชยวันสงกรานต์".
        output = self.th_exporter.generate()

        songkran_count = output.count("SUMMARY:วันสงกรานต์\r\n")
        self.assertEqual(songkran_count, 1)
        self.assertIn("DTSTART;VALUE=DATE:20240413\r\n", output)
        self.assertIn("DURATION:P3D\r\n", output)

    def test_single_holiday_multiple_date_continuous_overlap(self):
        # "День міжнародної солідарності трудящих" x2
        # but 2005-05-01 got "Великдень (Пасха)" as well.
        ua_holidays = country_holidays("UA", years=2005, language="uk")
        output = ICalExporter(ua_holidays).generate()

        labor_day_count = output.count("SUMMARY:День міжнародної солідарності трудя\r\n щих\r\n")
        self.assertEqual(labor_day_count, 1)
        self.assertIn("DTSTART;VALUE=DATE:20050501\r\n", output)
        self.assertIn("DURATION:P2D\r\n", output)

    def test_single_holiday_multiple_date_noncontinuous(self):
        # 2x "Eid al-Fitr", 2x "Eid al-Fitr Second Day".
        output = self.id_exporter.generate()

        eid_al_fitr_count = output.count("SUMMARY:Eid al-Fitr\r\n")
        self.assertEqual(eid_al_fitr_count, 2)
        eid_duration_count = output.count("DURATION:P1D\r\n")
        self.assertGreaterEqual(eid_duration_count, 2, "Expected at least 2 single-day events")

    def test_generate_from_combined_holidays_list(self):
        # All 3 "New Year's Day" should be merged into 1 single instance.
        cn_holidays = country_holidays("CN", years=2024, language="en_US")
        jp_holidays = country_holidays("JP", years=2024, language="en_US")
        kr_holidays = country_holidays("KR", years=2024, language="en_US")
        east_asia_holidays = cn_holidays + jp_holidays + kr_holidays
        output = ICalExporter(east_asia_holidays).generate()

        new_years_day_count = output.count("SUMMARY:New Year's Day\r\n")
        self.assertEqual(new_years_day_count, 1)
        self.assertIn("DTSTART;VALUE=DATE:20240101\r\n", output)

    def test_escape_character_holiday_names(self):
        # Default case, no action taken.
        self._assert_special_char_handling("Fête Nationale", "Fête Nationale")

        # SINGLE QUOTES, no action taken.
        self._assert_special_char_handling("Single 'Quotes' Holiday", "Single 'Quotes' Holiday")

        # DOUBLE QUOTES, no action taken.
        self._assert_special_char_handling('Double "Quotes" Holiday', 'Double "Quotes" Holiday')

        # COMMA.
        self._assert_special_char_handling(
            "Dia de Portugal, de Camões e das Comunidades Portuguesas",
            "Dia de Portugal\\, de Camões e das Comunidades Portuguesas",
        )

        # COLON.
        self._assert_special_char_handling("Special: Holiday Event", "Special\\: Holiday Event")

        # BACKSLASH.
        self._assert_special_char_handling("Backslash\\Holiday", "Backslash\\\\Holiday")

        # SEMICOLON is used as a delimiter in HolidayBase (HOLIDAY_NAME_DELIMITER = "; "),
        # so a name with a semicolon gets split into two separate `VEVENT`s.
        self._assert_special_char_handling("Christmas; Celebration", "Celebration")

    def test_localized_holiday_names(self):
        output = self.jp_exporter.generate()

        self.assertIn("SUMMARY:元日\r\n", output)

    def test_empty_holidays(self):
        exporter = ICalExporter(country_holidays("TH", years=1800))
        output = exporter.generate()

        # iCalendar File Header.
        self.assertIn("BEGIN:VCALENDAR", output)
        self.assertIn("PRODID:", output)
        self.assertIn("VERSION:2.0", output)
        self.assertIn("CALSCALE:GREGORIAN", output)
        self.assertIn("END:VCALENDAR", output)

        # Verify no `VEVENT`s are generated for empty holiday set.
        self.assertNotIn("BEGIN:VEVENT", output)
        self.assertNotIn("DTSTAMP:", output)
        self.assertNotIn("UID:", output)
        self.assertNotIn("SUMMARY:", output)
        self.assertNotIn("DTSTART;", output)
        self.assertNotIn("DURATION:", output)
        self.assertNotIn("END:VEVENT", output)

    def test_line_folding_edge_cases(self):
        # 74 octets.
        line_74_bytes = "x" * 66
        self._assert_line_lengths(line_74_bytes)

        # 75 octets.
        exact_75_bytes = "x" * 72 + "あ"
        self._assert_line_lengths(exact_75_bytes)

        # 76 octets.
        needs_adjustment = "x" * 73 + "あ"
        self._assert_line_lengths(needs_adjustment)

    def test_long_holiday(self):
        # ASCII-only
        ascii_fold_test = (
            "Krungthepmahanakhon Amonrattanakosin Mahintharayutthaya Mahadilokphop "
            "Noppharatratchathaniburirom Udomratchaniwetmahasathan Amonphimanawatansathit "
            "Sakkathattiyawitsanukamprasit"
        )
        self._assert_line_lengths(ascii_fold_test)

        # UTF-8
        utf8_fold_test = (
            "กรุงเทพมหานคร อมรรัตนโกสินทร์ มหินทรายุธยา มหาดิลกภพ นพรัตนราชธานีบูรีรมย์ "
            "อุดมราชนิเวศน์มหาสถาน อมรพิมานอวตารสถิต สักกะทัตติยวิษณุกรรมประสิทธิ์"
        )
        self._assert_line_lengths(utf8_fold_test)

        # Emojis test (Unicode codepoints for emojis)
        emoji_fold_test = "🎉🎊🎈🎁🎀✨🌟🌈🍀🐶🐱🦄🐯🐯🦄💥🌍🌎🌏🎶🎵🎧🎤🎬🎼🎷🎸"
        self._assert_line_lengths(emoji_fold_test)

        # Multi-codepoint graphemes (combinations of accents and other modifiers)
        grapheme_fold_test = "áèîöũčñ𝒜𝒩𝒲👩‍🚀👨‍🍳👩‍🔬👨‍🎤👩‍🎨👨‍🚒"
        self._assert_line_lengths(grapheme_fold_test)

    def test_return_bytes(self):
        self._assert_byte_output(self.us_exporter, b"SUMMARY:New Year's Day")
        self._assert_byte_output(self.jp_exporter, b"SUMMARY:\xe5\x85\x83\xe6\x97\xa5")

    def test_crlf_line_endings(self):
        output = self.th_exporter.generate()

        lines = output.splitlines(keepends=True)
        for line in lines:
            self.assertTrue(
                line.endswith(CONTENT_LINE_DELIMITER), f"Line did not end with CRLF: {repr(line)}"
            )

    def test_unique_uid_generation(self):
        # 1st generation should yield unique UUIDs for each VEVENT.
        output = self.us_exporter.generate()
        uids = [line for line in output.split(CONTENT_LINE_DELIMITER) if line.startswith("UID:")]

        self.assertEqual(len(uids), len(set(uids)), "Duplicate UIDs found in iCal output.")

        # 2nd generation should yield different UUIDs.
        output_2 = self.us_exporter.generate()
        uids_2 = [
            line for line in output_2.split(CONTENT_LINE_DELIMITER) if line.startswith("UID:")
        ]

        self.assertEqual(len(uids_2), len(set(uids_2)), "Duplicate UIDs found in 2nd iCal output.")

        # Ensure that there is no overlap at all between UIDs from both attempt.
        self.assertTrue(set(uids).isdisjoint(set(uids_2)), "Some UIDs are reused")

    def test_save_ics_valid_path(self):
        with tempfile.TemporaryDirectory() as valid_path:
            file_path = Path(valid_path) / "test_calendar.ics"

            self.us_exporter.save_ics(file_path=file_path)
            self.assertTrue(file_path.exists(), f"File should be created at {file_path}")

    def test_save_ics_pathlib_path(self):
        with tempfile.TemporaryDirectory() as valid_path:
            file_path_1 = Path(valid_path) / "test_calendar_1.ics"
            self.us_exporter.save_ics(file_path=file_path_1)
            content_1 = [
                line
                for line in file_path_1.read_text().splitlines()
                if not line.startswith(("UID:", "DTSTAMP:"))
            ]

            file_path_2 = Path(valid_path) / "test_calendar_2.ics"
            self.us_exporter.save_ics(file_path=str(file_path_2))
            content_2 = [
                line
                for line in file_path_2.read_text().splitlines()
                if not line.startswith(("UID:", "DTSTAMP:"))
            ]

            self.assertEqual(content_1, content_2)

    def test_save_ics_empty_content(self):
        self.exporter.generate = MagicMock(return_value=b"")

        with tempfile.NamedTemporaryFile(suffix=".ics") as temp_file:
            with self.assertRaises(ValueError) as context:
                self.exporter.save_ics(file_path=temp_file.name)
            self.assertEqual(str(context.exception), "Generated content is empty or invalid.")

    def test_save_ics_file_overwrite(self):
        with tempfile.NamedTemporaryFile(suffix=".ics", delete=False) as temp_file:
            temp_file.write(b"Old content")
            temp_file.close()

            self.us_exporter.save_ics(file_path=temp_file.name)
            new_content = Path(temp_file.name).read_text()

            self.assertNotEqual(new_content, "Old content", "The file was not overwritten.")
            self.assertIn("BEGIN:VCALENDAR", new_content, "New content is not valid iCalendar")
            Path(temp_file.name).unlink()

    def test_save_ics_with_utf8_name(self):
        with tempfile.NamedTemporaryFile(
            prefix="test_ปฏิทิน", suffix=".ics", delete=False
        ) as temp_file:
            temp_file.close()

            self.th_exporter.save_ics(file_path=temp_file.name)
            self.assertTrue(
                Path(temp_file.name).exists(),
                "File should be created with special characters in the name.",
            )
            Path(temp_file.name).unlink()

    def test_holidays_category(self):
        single_category_holidays = country_holidays(
            "US", years=2024, categories=UNOFFICIAL, language="en_US"
        )
        output = ICalExporter(single_category_holidays).generate()
        self.assertIn("CATEGORIES:UNOFFICIAL", output)

        multi_category_holidays = country_holidays(
            "US", years=2024, categories=(HALF_DAY, UNOFFICIAL), language="en_US"
        )
        output = ICalExporter(multi_category_holidays).generate()
        self.assertNotIn("CATEGORIES:HALF_DAY", output)
        self.assertNotIn("CATEGORIES:UNOFFICIAL", output)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.northern_mariana_islands import NorthernMarianaIslands
from tests.common import CommonCountryTests


class TestNorthernMarianaIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(NorthernMarianaIslands)

    def test_commonwealth_covenant_day(self):
        name = "Commonwealth Covenant Day"
        self.assertHolidayName(name, (f"{year}-03-24" for year in self.full_range))
        obs_dt = (
            "2012-03-23",
            "2013-03-25",
            "2018-03-23",
            "2019-03-25",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHolidayName(f"{name} (observed)", obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_commonwealth_cultural_day(self):
        name = "Commonwealth Cultural Day"
        self.assertHolidayName(
            name,
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(name, self.full_range)

    def test_election_day(self):
        name = "Election Day"
        self.assertHolidayName(
            name,
            "2008-11-04",
            "2010-11-02",
            "2012-11-06",
            "2014-11-04",
            "2016-11-08",
            "2018-11-06",
            "2020-11-03",
            "2022-11-08",
            "2024-11-05",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2008), range(2009, self.end_year, 2))

    def test_citizenship_day(self):
        name = "Citizenship Day"
        self.assertHolidayName(name, (f"{year}-11-04" for year in self.full_range))
        obs_dts = (
            "2012-11-05",
            "2017-11-03",
            "2018-11-05",
            "2023-11-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHolidayName(f"{name} (observed)", obs_dts)

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(name, (f"{year}-12-08" for year in self.full_range))
        obs_dts = (
            "2012-12-07",
            "2013-12-09",
            "2018-12-07",
            "2019-12-09",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHolidayName(f"{name} (observed)", obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-17", "Martin Luther King Jr. Day"),
            ("2022-02-14", "Valentine's Day"),
            ("2022-02-21", "Washington's Birthday"),
            ("2022-03-17", "Saint Patrick's Day"),
            ("2022-03-24", "Commonwealth Covenant Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-08", "Mother's Day"),
            ("2022-05-30", "Memorial Day"),
            ("2022-06-19", "Father's Day; Juneteenth National Independence Day"),
            ("2022-06-20", "Juneteenth National Independence Day (observed)"),
            ("2022-07-04", "Independence Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-10-10", "Commonwealth Cultural Day"),
            ("2022-10-31", "Halloween"),
            ("2022-11-04", "Citizenship Day"),
            ("2022-11-08", "Election Day"),
            ("2022-11-11", "Veterans Day"),
            ("2022-11-24", "Thanksgiving Day"),
            ("2022-12-08", "Constitution Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-17", "วันมาร์ติน ลูเทอร์ คิง จูเนียร์"),
            ("2022-02-14", "วันวาเลนไทน์"),
            ("2022-02-21", "วันเกิดวอชิงตัน"),
            ("2022-03-17", "วันนักบุญแพทริก"),
            ("2022-03-24", "วันปฏิญญาเครือรัฐ"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-05-08", "วันแม่"),
            ("2022-05-30", "วันรำลึก"),
            ("2022-06-19", "วันประกาศอิสรภาพแห่งชาติจูนทีนท์; วันพ่อ"),
            ("2022-06-20", "ชดเชยวันประกาศอิสรภาพแห่งชาติจูนทีนท์"),
            ("2022-07-04", "วันประกาศอิสรภาพ"),
            ("2022-09-05", "วันแรงงาน"),
            ("2022-10-10", "วันวัฒนธรรมแห่งเครือรัฐ"),
            ("2022-10-31", "วันฮาโลวีน"),
            ("2022-11-04", "วันแห่งความเป็นพลเมือง"),
            ("2022-11-08", "วันเลือกตั้ง"),
            ("2022-11-11", "วันทหารผ่านศึก"),
            ("2022-11-24", "วันขอบคุณพระเจ้า"),
            ("2022-12-08", "วันรัฐธรรมนูญ"),
            ("2022-12-24", "วันคริสต์มาสอีฟ"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "ชดเชยวันคริสต์มาส"),
            ("2022-12-31", "วันสิ้นปี"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.countries.france import France
from tests.common import CommonCountryTests


class TestFrance(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(France)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_2017(self):
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "Jour de l'an"),
            ("2017-04-17", "Lundi de Pâques"),
            ("2017-05-01", "Fête du Travail"),
            ("2017-05-08", "Fête de la Victoire"),
            ("2017-05-25", "Ascension"),
            ("2017-06-05", "Lundi de Pentecôte"),
            ("2017-07-14", "Fête nationale"),
            ("2017-08-15", "Assomption"),
            ("2017-11-01", "Toussaint"),
            ("2017-11-11", "Armistice"),
            ("2017-12-25", "Noël"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Jour de l'an"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-05-01", "Fête du Travail"),
            ("2024-05-08", "Fête de la Victoire"),
            ("2024-05-09", "Ascension"),
            ("2024-05-20", "Lundi de Pentecôte"),
            ("2024-07-14", "Fête nationale"),
            ("2024-08-15", "Assomption"),
            ("2024-11-01", "Toussaint"),
            ("2024-11-11", "Armistice"),
            ("2024-12-25", "Noël"),
        )

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_deprecated(self):
        self.assertEqual(
            sorted(France(subdiv="Alsace-Moselle", years=2023).keys()),
            sorted(France(subdiv="6AE", years=2023).keys()),
        )
        self.assertEqual(
            sorted(France(subdiv="Saint-Barthélémy", years=2023).keys()),
            sorted(France(subdiv="BL", years=2023).keys()),
        )

    def test_new_years_day(self):
        name = "Jour de l'an"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1811, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1811))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1886, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1886))

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertHolidayName(
            name,
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, range(1886, 2005), range(2008, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1886), range(2005, 2008))

    def test_labor_day(self):
        name_1919 = "1er mai"
        name_1941 = "Fête du Travail et de la Concorde sociale"
        name_1948 = "Fête du Travail"
        self.assertHolidayName(
            name_1919, (f"{year}-05-01" for year in (*range(1919, 1941), *range(1946, 1948)))
        )
        self.assertHolidayName(name_1941, (f"{year}-05-01" for year in range(1941, 1946)))
        self.assertHolidayName(name_1948, (f"{year}-05-01" for year in range(1948, self.end_year)))
        self.assertNoHolidayName(
            name_1919, range(self.start_year, 1919), range(1941, 1946), range(1948, self.end_year)
        )
        self.assertNoHolidayName(
            name_1941, range(self.start_year, 1941), range(1946, self.end_year)
        )
        self.assertNoHolidayName(name_1948, range(self.start_year, 1948))

    def test_victory_day(self):
        name = "Fête de la Victoire"
        self.assertHolidayName(
            name, (f"{year}-05-08" for year in (*range(1953, 1960), *range(1982, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1953), range(1960, 1982))

    def test_national_day(self):
        name = "Fête nationale"
        self.assertHolidayName(name, (f"{year}-07-14" for year in range(1880, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1880))

    def test_ascension_day(self):
        name = "Ascension"
        self.assertHolidayName(
            name,
            "2019-05-30",
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName("Assomption", (f"{year}-08-15" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName("Toussaint", (f"{year}-11-01" for year in self.full_range))

    def test_armistice_day(self):
        name = "Armistice"
        self.assertHolidayName(name, (f"{year}-11-11" for year in range(1922, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1922))

    def test_christmas_day(self):
        self.assertHolidayName("Noël", (f"{year}-12-25" for year in self.full_range))

    def test_good_friday(self):
        name = "Vendredi saint"
        dt = (
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                # Alsace, Moselle - 1893.
                case "57" | "6AE":
                    self.assertHolidayName(name, holidays, dt)
                    self.assertHolidayName(name, holidays, range(1893, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1893))
                # Guadeloupe, Martinique, French Polynesia - Unknown, but exists.
                case "971" | "972" | "PF":
                    self.assertHolidayName(name, holidays, dt)
                    self.assertHolidayName(name, holidays, self.full_range)
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_saint_stephens_day(self):
        name = "Saint Étienne"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Alsace, Moselle - 1892.
            if subdiv in {"57", "6AE"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-26" for year in range(1892, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1892))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_mi_careme(self):
        name = "Mi-Carême"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            dt = (
                "2019-03-28",
                "2020-03-19",
                "2021-03-11",
                "2022-03-24",
                "2023-03-16",
                "2024-03-07",
                "2025-03-27",
            )
            # Guadeloupe - Unknown, but exists.
            if subdiv == "971":
                self.assertHolidayName(name, holidays, dt)
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_abolition_of_slavery(self):
        name = "Abolition de l'esclavage"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                # Guadeloupe - 1984.
                case "971":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-05-27" for year in range(1984, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
                # Martinique - 1984.
                case "972":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-05-22" for year in range(1984, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
                # Guyane - 1984.
                case "973":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-06-10" for year in range(1984, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
                # Reunion - 1983.
                case "974":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-12-20" for year in range(1983, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
                # Mayotte - 1984.
                case "976":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-04-27" for year in range(1984, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
                # Saint Barthelemy - 2012.
                case "BL":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-10-09" for year in range(2012, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2012))
                # Saint Martin - 2012.
                case "MF":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-05-28" for year in range(2012, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2012))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_victor_schoelcher_day(self):
        name = "Fête de Victor Schoelcher"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                # Guadeloupe, Martinique - 1984.
                case "971" | "972":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-07-21" for year in range(1984, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
                # Saint Martin - 2012.
                case "MF":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-07-21" for year in range(2012, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(self.start_year, 2012))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_citizenship_day(self):
        name_1953 = "Fête de la prise de possession"
        name_2004 = "Fête de la Citoyenneté"
        self.assertNoHolidayName(name_1953)
        self.assertNoHolidayName(name_2004)
        for subdiv, holidays in self.subdiv_holidays.items():
            # New Caledonia - 1953.
            if subdiv == "NC":
                self.assertHolidayName(
                    name_1953, holidays, (f"{year}-09-24" for year in range(1953, 2004))
                )
                self.assertHolidayName(
                    name_2004, holidays, (f"{year}-09-24" for year in range(2004, self.end_year))
                )
                self.assertNoHolidayName(
                    name_1953, holidays, range(self.start_year, 1953), range(2004, self.end_year)
                )
                self.assertNoHolidayName(name_2004, holidays, range(self.start_year, 2004))
            else:
                self.assertNoHolidayName(name_1953, holidays)
                self.assertNoHolidayName(name_2004, holidays)

    def test_missionary_day(self):
        name = "Arrivée de l'Évangile"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # French Polynesia - 1978.
            if subdiv == "PF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-05" for year in range(1978, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1978))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_internal_autonomy_day(self):
        name = "Fête de l'autonomie"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # French Polynesia - 1985-2024.
            if subdiv == "PF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-29" for year in range(1985, 2025))
                )
                self.assertNoHolidayName(
                    name, holidays, range(self.start_year, 1985), range(2025, self.end_year)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_matarii(self):
        name = "Matāri'i"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # French Polynesia - 2025.
            if subdiv == "PF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-20" for year in range(2025, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2025))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_feast_of_saint_peter_chanel(self):
        name = "Saint Pierre Chanel"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Wallis and Futuna - 1962.
            if subdiv == "WF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-28" for year in range(1962, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1962))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saints_peter_and_paul_day(self):
        name = "Saints Pierre et Paul"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Wallis and Futuna - 1962.
            if subdiv == "WF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-29" for year in range(1962, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1962))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_territory_day(self):
        name = "Fête du Territoire"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            # Wallis and Futuna - 1962.
            if subdiv == "WF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-29" for year in range(1962, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1962))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Jour de l'an"),
            ("2022-03-05", "Arrivée de l'Évangile"),
            ("2022-03-24", "Mi-Carême"),
            ("2022-04-15", "Vendredi saint"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-04-27", "Abolition de l'esclavage"),
            ("2022-04-28", "Saint Pierre Chanel"),
            ("2022-05-01", "Fête du Travail"),
            ("2022-05-08", "Fête de la Victoire"),
            ("2022-05-22", "Abolition de l'esclavage"),
            ("2022-05-26", "Ascension"),
            ("2022-05-27", "Abolition de l'esclavage"),
            ("2022-05-28", "Abolition de l'esclavage"),
            ("2022-06-06", "Lundi de Pentecôte"),
            ("2022-06-10", "Abolition de l'esclavage"),
            ("2022-06-29", "Fête de l'autonomie; Saints Pierre et Paul"),
            ("2022-07-14", "Fête nationale"),
            ("2022-07-21", "Fête de Victor Schoelcher"),
            ("2022-07-29", "Fête du Territoire"),
            ("2022-08-15", "Assomption"),
            ("2022-09-24", "Fête de la Citoyenneté"),
            ("2022-10-09", "Abolition de l'esclavage"),
            ("2022-11-01", "Toussaint"),
            ("2022-11-11", "Armistice"),
            ("2022-12-20", "Abolition de l'esclavage"),
            ("2022-12-25", "Noël"),
            ("2022-12-26", "Saint Étienne"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-03-05", "Missionary Day"),
            ("2022-03-24", "Mi-Careme"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "Abolition of Slavery"),
            ("2022-04-28", "Feast of Saint Peter Chanel"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-08", "Victory Day"),
            ("2022-05-22", "Abolition of Slavery"),
            ("2022-05-26", "Ascension Day"),
            ("2022-05-27", "Abolition of Slavery"),
            ("2022-05-28", "Abolition of Slavery"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-10", "Abolition of Slavery"),
            ("2022-06-29", "Internal Autonomy Day; Saints Peter and Paul Day"),
            ("2022-07-14", "National Day"),
            ("2022-07-21", "Victor Schoelcher Day"),
            ("2022-07-29", "Territory Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-09-24", "Citizenship Day"),
            ("2022-10-09", "Abolition of Slavery"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-11", "Armistice Day"),
            ("2022-12-20", "Abolition of Slavery"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-03-05", "วันคริสตธรรมมาถึง"),
            ("2022-03-24", "วันเฉลิมฉลองกลางเทศกาลมหาพรต"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-04-27", "วันเลิกทาส"),
            ("2022-04-28", "วันสมโภชนักบุญเปโตร ชาเนล"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-08", "วันแห่งชัยชนะ"),
            ("2022-05-22", "วันเลิกทาส"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-05-27", "วันเลิกทาส"),
            ("2022-05-28", "วันเลิกทาส"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-06-10", "วันเลิกทาส"),
            ("2022-06-29", "วันปกครองตนเอง; วันสมโภชนักบุญเปโตรและเปาโล"),
            ("2022-07-14", "วันชาติฝรั่งเศส"),
            ("2022-07-21", "วันวิกตอร์ เชลแชร์"),
            ("2022-07-29", "วันก่อตั้งดินแดน"),
            ("2022-08-15", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2022-09-24", "วันแห่งความเป็นพลเมือง"),
            ("2022-10-09", "วันเลิกทาส"),
            ("2022-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2022-11-11", "วันสงบศึก"),
            ("2022-12-20", "วันเลิกทาส"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "วันสมโภชนักบุญสเตเฟน"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-03-05", "День місіонерів"),
            ("2022-03-24", "Свято Мі-Карем"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-27", "День скасування рабства"),
            ("2022-04-28", "День Святого Пʼєра Шанеля"),
            ("2022-05-01", "День праці"),
            ("2022-05-08", "День Перемоги"),
            ("2022-05-22", "День скасування рабства"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-05-27", "День скасування рабства"),
            ("2022-05-28", "День скасування рабства"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-10", "День скасування рабства"),
            ("2022-06-29", "День Святих Петра і Павла; День автономії"),
            ("2022-07-14", "Національне свято"),
            ("2022-07-21", "День Віктора Шольшера"),
            ("2022-07-29", "День Території"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-09-24", "День громадянства"),
            ("2022-10-09", "День скасування рабства"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-11", "День перемирʼя"),
            ("2022-12-20", "День скасування рабства"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День Святого Стефана"),
        )
